from datetime import timedelta, timezone, datetime
import pytz
from utilspkg import utils_init

if __name__ == '__main__':
    utils_init.load_env_variables_from_yaml('/Users/croft/VScode/ptagit/env_vars.yaml')

logger = utils_init.setup_logger(__name__)



def get_datetime_from_timezone(timezone_name='US/Eastern'):
    """
    Returns the current datetime object of a specific timezone.

    Args:
        timezone_name (str, optional): The timezone you want to get the current time from. Defaults to 'US/Eastern'.
    
    Returns:
        datetime: Current datetime object of the provided timezone.
    """
    tz = pytz.timezone(timezone_name)
    return datetime.now(tz)


def get_day_name_from_dt(date_time):
    """
    Returns the day of the week from a datetime object.

    Args:
        date_time (datetime): Datetime object to get the day of the week from.
    
    Returns:
        str: Day of the week.
    """
    return date_time.strftime('%A')


def get_time_string1(date_time):    
    """
    Returns a string like "Wednesday 15:00"

    Args:
        date_time (datetime): Datetime object to get the day and hour from.       
    """
    time_str = f'{get_day_name_from_dt(date_time=date_time)}, {date_time.hour:02d}:00'

    return time_str


def get_time_string2(date_time):    
    """
    Returns a formatted string containing the day of the week and the time.

    Args:
        date_time (datetime): Datetime object to get the day and time from.
    
    Returns:
        str: Formatted string.
    """
    return date_time.strftime('%A, %H:%M')


def get_time_string3(date_time):    
    """
    Returns a string like: "Wednesday May 17"

    Args:
        date_time (datetime): Datetime object to get the day, month, and day from.
    
    Returns:
        str: Formatted string.
    """
    return date_time.strftime("%A %B %d")


def get_datetime_from_slack_timestamp(slack_timestamp, tz_offset=0):
    """
    Converts a Slack timestamp and timezone offset into a datetime object.

    Args:
        slack_timestamp (str): Slack timestamp to convert.
        tz_offset (int, optional): Timezone offset in seconds. Defaults to 0.
    
    Returns:
        datetime: Corresponding datetime object.
    """
    # Convert Slack timestamp to Unix timestamp (in seconds) or use current time
    if slack_timestamp:
        unix_timestamp = float(slack_timestamp.split('.')[0])
    else:
        unix_timestamp = datetime.now().timestamp()

    dt = datetime.fromtimestamp(unix_timestamp, timezone.utc)  # timezone-aware datetime
    return dt + timedelta(seconds=tz_offset)  



def get_datetime_from_slack_tz_offset(tz_offset, date_time=None):
    """
    Adjusts a datetime object with a Slack timezone offset.

    Args:
        tz_offset (int, optional): Timezone offset in seconds. Defaults to 0.
        date_time (datetime, optional): Datetime object to adjust. If not provided, defaults to current UTC datetime.
    
    Returns:
        datetime: Adjusted datetime object.
    """
    date_time = date_time if date_time else datetime.now(timezone.utc)
    return date_time + timedelta(seconds=tz_offset)



import os
import logging
from airtable import Airtable

from utilspkg import utils_init

if __name__ == '__main__':
    utils_init.load_env_variables_from_yaml('/Users/croft/VScode/ptagit/env_vars.yaml')

logger = utils_init.setup_logger(__name__)
# logger = logging.getLogger(__name__)

class DBConnect:
    """Class to interact with Airtable API"""

    def __init__(self, base_id=None, api_key=None, table_name=""):
        # Load environment variables if not provided
        self.base_id = base_id if base_id else os.environ['AIRTABLE_BASE_KEY']
        self.api_key = api_key if api_key else os.environ['AIRTABLE_API_KEY']
        self.table_name = table_name

    def get_table_object(self, table_name=""):
        """Return the Airtable object for given table_name"""
        
        # If we received the tablename argument, use that, else use the self variable
        table_name = table_name if table_name else self.table_name

        if not table_name:
            raise ValueError("table_name must be provided")

        try:
            return Airtable(base_id=self.base_id, table_name=table_name, api_key=self.api_key)
        except Exception as e:
            logger.error(f"Error in getting table object: {e}")
            raise

    def helper_table_name_or_object (self, table_name_or_object):
        '''Class helper function to test if the user sent a table name or an object. Returns a table object '''
        if isinstance(table_name_or_object, str): #we got a table name 'string'
            table_object = self.get_table_object(table_name_or_object)
        
        elif table_name_or_object is not None: # we think we got a table object
            table_object = table_name_or_object
        
        else:
            table_object = None
            raise ValueError("table_name_or_object must be provided")
        
        return table_object

    def get_records (self, table_name_or_object, **kwargs):
        """Returns table records. Optional kwargs:
         - formula=str,
         - sort=[], 
         - maxrecords=#, 
         """
        kwargs = {key: value for key, value in kwargs.items() if value is not None}

        table_object = self.helper_table_name_or_object(table_name_or_object)

        return table_object.get_all (**kwargs)
    
    
    def get_record (self, table_name_or_object, record_id):
        """
        The get_record function retrieves a record from the Airtable API.
        :param table_name_or_object: Determine whether the table name or object is being passed in
        :param record_id: Specify which record to retrieve
        :return: A record from a table
        """

        table_object = self.helper_table_name_or_object(table_name_or_object)

        return table_object.get(record_id)


    def search_table(self, table_name_or_object, field_name, search_text):
        """Search for records in a table with a specific field matching the search_text"""
        table_object = self.helper_table_name_or_object(table_name_or_object)

        return table_object.search(field_name, search_text)

    def update_record(self, table_name_or_object, record_id, record_dict):
        '''Runs table.update(recordid, recordfields) on table_name_or_object'''
        table_object = self.helper_table_name_or_object(table_name_or_object)
        
        return table_object.update(record_id=record_id, fields=record_dict)


    def insert_record(self, table_name_or_object, record_dict):
        '''Runs table.insert(record_dict) on table_name_or_object'''
        table_object = self.helper_table_name_or_object(table_name_or_object)

        try:
            return table_object.insert(fields=record_dict)
        except Exception as e:
            logger.error(f"Error in inserting record: {e}")
            raise



# Utils to load env variables and set up logging
from dotenv import load_dotenv #used locally in files
import logging
import yaml
import os
import sys
# Imports the Cloud Logging client library
# import google.cloud.logging

# print("\n******************\n\nTURNS OUT -E WORKED FOR PIP INSTALL!!!\n\n******************")
LOG_FORMAT = "%(levelname)s [function: %(funcName)s] - %(message)s"
ENV_FULL_PATH_IF_NULL = "/Users/croft/VScode/ptagit/env_vars.yaml"

# Check if code is running in VS Code debugger locally. If so, set TESTING_FLAG to 1 and don't exclude my slack id
if sys.gettrace() is not None:
    TESTING_FLAG = 1 
    # set testingflag as an env variable
    os.environ["TESTING_FLAG"] = str(TESTING_FLAG)
    print("\n******************\n\nRUNNING IN VS CODE DEBUGGER\n\n****************** (UTILS_INIT.PY)")
    EXCLUDED_SLACK_IDS = []

def setup_logger(logger_name, log_level=logging.INFO, log_format=LOG_FORMAT):
    logger = logging.getLogger(logger_name)
    logger.setLevel(log_level)
    print(f"logger name: {logger_name}")

    # add console handler with custom format if running standalone
    # if logger_name == '__main__' and not logger.handlers:
    console_handler = logging.StreamHandler()
    console_handler.setLevel(log_level)  # Set the log level on the handler
    formatter = logging.Formatter(log_format)
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)
    # print(f"Adding handler to logger: {logger_name}")

    return logger

def load_env_variables_from_yaml(full_path_file_name=ENV_FULL_PATH_IF_NULL):    
    if hasattr(load_env_variables_from_yaml, "has_run"):
        return
    try:
        with open(full_path_file_name, 'r') as stream:
            params = yaml.safe_load(stream)
            for key, value in params.items():
                value = str(value) 
                os.environ[key] = value
        # set this attribute so don't keep running the function in the same program
        load_env_variables_from_yaml.has_run = True
    except FileNotFoundError:
        pass  # Ignore the file not found error as it is expected in some environments
            


# SCRAP:

    # Instantiates a client
    # client = google.cloud.logging.Client()

    # Retrieves a Cloud Logging handler based on the environment
    # you're running in and integrates the handler with the
    # Python logging module. By default this captures all logs
    # at INFO level and higher
    # client.setup_logging()

## COULD MAYBE DELETE SINCE SINCE LOAD_DOTENV SHOWS UP AS A METHOD WITH MORE OPTIONS WHEN USING THIS UTILS_INIT!!!
# def DELETE_load_dotenv_file(path = "../.env"):
    # load_dotenv(dotenv_path=path)



import re
from typing import Optional


def extract_integer(text: str, round: bool = False) -> Optional[int]:
    """
    Extracts the first number from the provided text and returns it as an integer.
    If no number is found, returns None.
    If 'round' is True, rounds the number to the nearest integer before returning.
    """
    number = extract_float(text)
    if number is not None:
        return int(round(number)) if round else int(number)
    else:
        return None


def extract_float(text: str) -> Optional[float]:
    """
    Searches for the first number in the provided text and returns it as a float.
    If no number is found, returns None.
    """
    match = re.search(r'\b\d+\b', text)
    return float(match.group()) if match else None



from datetime import timedelta, timezone, datetime
import pytz

class UtilsTime:
    ''' Set of functions to return useful datetime objects and strings from 
    slack timezone offsets, slack timestamps, and more'''

    @staticmethod
    def get_datetime_from_timezone(timezone_name='US/Eastern'):
        """
        Returns the current datetime object of a specific timezone.

        Args:
            timezone_name (str, optional): The timezone you want to get the current time from. Defaults to 'US/Eastern'.
        
        Returns:
            datetime: Current datetime object of the provided timezone.
        """
        tz = pytz.timezone(timezone_name)
        return datetime.now(tz)

    @staticmethod
    def get_day_name_from_dt(date_time):
        """
        Returns the day of the week from a datetime object.

        Args:
            date_time (datetime): Datetime object to get the day of the week from.
        
        Returns:
            str: Day of the week.
        """
        return date_time.strftime('%A')

    @staticmethod
    def get_time_string1(date_time):    
        """
        Returns a string like "Wednesday 15:00"

        Args:
            date_time (datetime): Datetime object to get the day and hour from.       
        """
        time_str = f'{UtilsTime.get_day_name_from_dt(date_time=date_time)}, {date_time.hour:02d}:00'

        return time_str

    @staticmethod
    def get_time_string2(date_time):    
        """
        Returns a formatted string containing the day of the week and the time.

        Args:
            date_time (datetime): Datetime object to get the day and time from.
        
        Returns:
            str: Formatted string.
        """
        return date_time.strftime('%A, %H:%M')

    @staticmethod
    def get_time_string3(date_time):    
        """
        Returns a string like: "Wednesday May 17"

        Args:
            date_time (datetime): Datetime object to get the day, month, and day from.
        
        Returns:
            str: Formatted string.
        """
        return date_time.strftime("%A %B %d")

    @staticmethod
    def get_datetime_from_slack_timestamp(slack_timestamp, tz_offset=0):
        """
        Converts a Slack timestamp and timezone offset into a datetime object.

        Args:
            slack_timestamp (str): Slack timestamp to convert.
            tz_offset (int, optional): Timezone offset in seconds. Defaults to 0.
        
        Returns:
            datetime: Corresponding datetime object.
        """
        unix_timestamp = float(slack_timestamp.split('.')[0])
        dt = datetime.fromtimestamp(unix_timestamp, timezone.utc)  # timezone-aware datetime
        return dt + timedelta(seconds=tz_offset)  


    @staticmethod
    def get_datetime_from_slack_tz_offset(tz_offset, date_time=None):
        """
        Adjusts a datetime object with a Slack timezone offset.

        Args:
            tz_offset (int, optional): Timezone offset in seconds. Defaults to 0.
            date_time (datetime, optional): Datetime object to adjust. If not provided, defaults to current UTC datetime.
        
        Returns:
            datetime: Adjusted datetime object.
        """
        date_time = date_time if date_time else datetime.now(timezone.utc)
        return date_time + timedelta(seconds=tz_offset)



from setuptools import setup, find_packages

setup(
    name='utilspkg',
    version='0.4.3',
    # this tells it to just use the utils/ folder
    packages=find_packages(include=['utilspkg']),
    # packages=find_packages(),
    # packages=find_packages(include=['utils_pkg', 'utils_pkg.utils']),
    requires=['airtable_python_wrapper', 'openai', 'python_dotenv', 'pytz', 'PyYAML', 'slack_sdk', 'tenacity']
    
)
